////////////////////////////////////////////////////////////////
//                    ""                 //
//                          1.0                         //
//             2018 (c)                //
//                  e-mail: sgsidorov@mail.ru                 //
////////////////////////////////////////////////////////////////

unit UFlowChart;

interface

uses  StdCtrls, ExtCtrls, Types, Controls, Classes, Graphics;

const  GridStep = 10;       //  

type
     TElem  = class         //     -
     private
       FImage  :TImage;                                          //   
       FPoints :array of TPoint;                                 //  
       FPoint  :longint;                                         //   
       FColor  :TColor;                                          //  
       FText   :string;                                          //   
       FLinks  :string[4];                                       // : 0-, 1-, 2-
       FRect   :TRect;                                           //  
       FWidth  :longint;                                         //  
       FHeight :longint;                                         //  
       function Xc:longint;                                      //    
       function Yc:longint;                                      //    
       function Frame(P:TPoint):TRect;                           //     P
       function FixValue(X:longint):longint;                     //   X  
       function NumPoint(P:TPoint):longint;                      //   
       function Include(P:TPoint):boolean;                       //    P  
       procedure SetPoints;                   virtual;           //  
       procedure SetLeft(X:longint);                             //   
       procedure SetTop(Y:longint);                              //   
       procedure SetWidth(W:longint);                            //  
       procedure SetHeight(H:longint);                           //  
       procedure SetText(S:string);                              //  
       procedure FixGrid;                                        //   
       procedure ShowSelect;                                     //  
       procedure Line(X1,Y1,X2,Y2:longint);                      //  
       procedure Arrow(X1,Y1,X2,Y2:longint);                     //  
       procedure Swap(var A,B:longint);                          //   A  B
       procedure ShowLinks;                                      //  
       procedure ShowContour;                 virtual;           //  
       procedure ShowText;                                       //  
     public
       constructor Create(I:TImage);                             //  
       destructor Destroy;                    override;          //  
       procedure Show;                                           //  
       property Image  :TImage  read FImage     write FImage;    // 
       property Left   :longint read FRect.Left write SetLeft;   //  
       property Top    :longint read FRect.Top  write SetTop;    //  
       property Width  :longint read FWidth     write SetWidth;  // 
       property Height :longint read FHeight    write SetHeight; // 
       property Text   :string  read FText      write SetText;   //   
     end;

     TBlockBegin = class(TElem)     //   ""
     private
       procedure ShowContour; override;       //   
     public
       constructor Create(I:TImage);          //  
     end;

     TBlockEnd = class(TElem)       //   ""
     private
       procedure ShowContour; override;       //   
     public
       constructor Create(I:TImage);          //  
     end;

     TBlockInput = class(TElem)     //   ""
     private
       procedure SetPoints; override;         //  
     public
       constructor Create(I:TImage);          //  
     end;

     TBlockAction = class(TElem)    //   ""
     private
     public
       constructor Create(I:TImage);          //  
     end;

     TBlockIf = class(TElem)        //   ""
     private
       procedure SetPoints; override;         //  
       procedure ShowContour; override;       //   
     public
       constructor Create(I:TImage);          //  
     end;

     TBlockCircle = class(TElem)    //   ""
     private
       procedure SetPoints; override;         //  
     public
       constructor Create(I:TImage);          //  
     end;

     THorLink = class(TElem)        //   
     private
       procedure SetPoints; override;         //  
       procedure ShowContour; override;       //  
     public
       constructor Create(I:TImage);          //  
     end;

     TVerLink = class(TElem)        //   
     private
       procedure SetPoints; override;         //  
       procedure ShowContour; override;       //  
     public
       constructor Create(I:TImage);          //  
     end;


     TMoveMode = (mdNone, mdMove, mdSize, mdChart);  //  : , , .,  


     TFlowChart = class     //   -
     private
       FElems    :array of TElem;               //  -
       FImage    :TImage;                       //   
       FCurElem  :TElem;                        //  
       FDX,FDY   :longint;                      //    
       FMode     :TMoveMode;                    //  
       FEdit     :TEdit;                        //    
       function FindElem(X,Y:longint):TElem;    //  
       procedure TextChange(Sender: TObject);   //  
       procedure FixGrid;                       //  -  
       procedure MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); //  
       procedure MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);                       // 
       procedure MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);   //  
       procedure SetImage(I:TImage);            //  
       procedure SetEdit(E:TEdit);              //     
     public
       procedure Clear;                         //  -
       constructor Create;                      //  -
       destructor Destroy; override;            //  -
       procedure Show;                          //  -
       procedure Add(B:TElem);                  //  
       procedure DelSelect;                     //   
       procedure NextMode;                      //    
       property Image:TImage write SetImage;    // 
       property Edit :TEdit  write SetEdit;     //    
     end;

var  FlowChart :TFlowChart;   // -

implementation

uses  Windows;

////////////////////////////////////////////////////////////////
//             - TElem           //
////////////////////////////////////////////////////////////////

function TElem.Xc:longint;  //    
begin
     result:=(FRect.Left+FRect.Right) div 2;
end;

function TElem.Yc:longint;  //    
begin
     result:=(FRect.Top+FRect.Bottom) div 2;
end;

function TElem.Frame(P:TPoint):TRect;  //     P
const  Size = 3;  // 
begin
     Result.Left:=P.X-Size;   Result.Right :=P.X+Size;
     Result.Top :=P.Y-Size;   Result.Bottom:=P.Y+Size;
end;

function TElem.FixValue(X:longint):longint;  //  X  
begin
     result:=Round(X/GridStep)*GridStep;
end;

function TElem.NumPoint(P:TPoint):longint;  //   
var  i :longint;  // 
begin
     result:=-1;
     for i:=0 to Length(FPoints)-1 do
         if PtInRect(Frame(FPoints[i]),P) then result:=i;
     FPoint:=result;
end;

function TElem.Include(P:TPoint):boolean;  //    P  
begin
     result:=PtInRect(FRect,P);
end;

procedure TElem.SetPoints;  //  
begin
     FPoints[0].X:=FRect.Left;    FPoints[0].Y:=FRect.Top;
     FPoints[1].X:=FRect.Right;   FPoints[1].Y:=FRect.Top;
     FPoints[2].X:=FRect.Right;   FPoints[2].Y:=FRect.Bottom;
     FPoints[3].X:=FRect.Left;    FPoints[3].Y:=FRect.Bottom;
end;

procedure TElem.SetLeft(X:longint);  //   
begin
     FRect.Left:=X;  FRect.Right:=FRect.Left+FWidth;  SetPoints;
end;

procedure TElem.SetTop(Y:longint);  //   
begin
     FRect.Top:=Y;  FRect.Bottom:=FRect.Top+FHeight;  SetPoints;
end;

procedure TElem.SetWidth(W:longint);  //  
begin
     if W<2*GridStep then W:=2*GridStep;
     FWidth:=W;  FRect.Right:=FRect.Left+W;  SetPoints;
end;

procedure TElem.SetHeight(H:longint);  //  
begin
     if H<2*GridStep then H:=2*GridStep;
     FHeight:=H;  FRect.Bottom:=FRect.Top+H;  SetPoints;
end;

procedure TElem.SetText(S:string);  //  
begin
     FText:=S;  Show;
end;

procedure TElem.FixGrid;  //   
begin
     if FRect.Left>FRect.Right then Swap(FRect.Left,FRect.Right);
     if FRect.Top>FRect.Bottom then Swap(FRect.Top,FRect.Bottom);
     Left:=FixValue(Left);            Top:=FixValue(Top);
     if Self is TVerLink then begin
        Width:=FixValue(Width div 2)*2;  Height:=FixValue(Height);
     end else if Self is THorLink then begin
        Width:=FixValue(Width);  Height:=FixValue(Height div 2)*2;
     end else begin
        Width:=FixValue(Width div 2)*2;  Height:=FixValue(Height div 2)*2;
     end;
end;

procedure TElem.ShowSelect;  //  
var  i :longint;  // 
begin
     FImage.Canvas.Brush.Color:=clBlue;
     for i:=0 to Length(FPoints)-1 do
         FImage.Canvas.FillRect(Frame(FPoints[i]));
end;

procedure TElem.Line(X1,Y1,X2,Y2:longint);  //  
begin
     FImage.Canvas.MoveTo(X1,Y1); FImage.Canvas.LineTo(X2,Y2);
end;

procedure TElem.Arrow(X1,Y1,X2,Y2:longint);  //  
const  W=3; L=7;  //    
begin
     Line(X1,Y1,X2,Y2);
     if X1=X2 then begin  //  
        if Y1<Y2 then begin Line(X2-W,Y2-L,X2,Y2); Line(X2+W,Y2-L,X2,Y2); end  //  
                 else begin Line(X2-W,Y2+L,X2,Y2); Line(X2+W,Y2+L,X2,Y2); end; //  
     end else begin       //  
        if X1<X2 then begin Line(X2-L,Y2-W,X2,Y2); Line(X2-L,Y2+W,X2,Y2); end  //  
                 else begin Line(X2+L,Y2-W,X2,Y2); Line(X2+L,Y2+W,X2,Y2); end; //  
     end;
end;

procedure TElem.Swap(var A,B:longint);  //   A  B
var  V :longint;  // 
begin
     V:=A;  A:=B;  B:=V;
end;

procedure TElem.ShowLinks;  //  
const  L=10;  //   
begin
     FImage.Canvas.Pen.Color:=RGB($00,$00,$00);
     if (FLinks[1]='1') or (FLinks[1]='3') then Arrow(FRect.Left-L,Yc,FRect.Left,Yc);      //  
     if (FLinks[1]='2') or (FLinks[1]='4') then Line(FRect.Left-L,Yc,FRect.Left,Yc);       //  
     if (FLinks[2]='1') or (FLinks[2]='3') then Arrow(Xc,FRect.Top-L,Xc,FRect.Top);        //  
     if (FLinks[2]='2') or (FLinks[2]='4') then Line(Xc,FRect.Top-L,Xc,FRect.Top);         //  
     if (FLinks[3]='1') or (FLinks[3]='3') then Arrow(FRect.Right+L,Yc,FRect.Right,Yc);    //  
     if (FLinks[3]='2') or (FLinks[3]='4') then Line(FRect.Right,Yc,FRect.Right+L,Yc);     //  
     if (FLinks[4]='1') or (FLinks[4]='3') then Arrow(Xc,FRect.Bottom+L,Xc,FRect.Bottom);  //  
     if (FLinks[4]='2') or (FLinks[4]='4') then Line(Xc,FRect.Bottom,Xc,FRect.Bottom+L);   //  
end;

procedure TElem.ShowContour;  //   
begin
     FImage.Canvas.Pen.Color:=RGB($00,$00,$00);
     FImage.Canvas.Brush.Color:=FColor;
     FImage.Canvas.Polygon(FPoints);
end;

procedure TElem.ShowText;  //   
var  X,Y :longint;  //  
begin
     X:=FRect.Left+((FRect.Right-FRect.Left-FImage.Canvas.TextWidth(FText)) div 2);
     Y:=FRect.Top+((FRect.Bottom-FRect.Top-FImage.Canvas.TextHeight(FText)) div 2);
     FImage.Canvas.Brush.Color:=FColor;
     FImage.Canvas.TextOut(X,Y,FText);
end;

////////////////////////////////////////////////////////////////

constructor TElem.Create(I:TImage);  //  
begin
     FImage  := I;                  // 
     SetLength(FPoints,4);          //  
     FWidth  := 100;                //  
     FHeight := 40;                 //  
     Left    := 10;                 //  
     Top     := 10;                 //  
     FPoint  := -1;                 //   
     FColor  := RGB($00,$00,$00);   //  
     FText   := '';                 //  
     FLinks  := '0102';             // :  ,  
end;

destructor TElem.Destroy;  //  
begin
     SetLength(FPoints,0);    //  
end;

procedure TElem.Show;  //  
begin
     if FImage=nil then Exit;
     ShowLinks;               //  
     ShowContour;             //  
     ShowText;                //  
end;


////////////////////////////////////////////////////////////////
//                 "" TBlockBegin             //
////////////////////////////////////////////////////////////////

constructor TBlockBegin.Create(I:TImage);  //  
begin
     inherited;                                     //  TElem.Create
     FLinks:='0002';                                //  
     FColor:=RGB($E5,$AA,$70);                      //  
     if FImage=nil then Exit;
     Left  := (FImage.ClientWidth-FWidth) div 2;    //  
     Text  := '';                             // 
end;

procedure TBlockBegin.ShowContour;  //   
begin
     FImage.Canvas.Pen.Color:=RGB($00,$00,$00);
     FImage.Canvas.Brush.Color:=FColor;
     FImage.Canvas.RoundRect(FRect.Left,FRect.Top,FRect.Right,FRect.Bottom,FHeight,FHeight);
end;


////////////////////////////////////////////////////////////////
//                  "" TBlockEnd               //
////////////////////////////////////////////////////////////////

constructor TBlockEnd.Create(I:TImage);  //  
begin
     inherited;                                     //  TElem.Create
     FLinks:='0100';                                //  
     FColor:=RGB($E5,$AA,$70);                      //  
     if FImage=nil then Exit;
     Left  := (FImage.ClientWidth-FWidth) div 2;    //  
     Top   := FImage.ClientHeight-FHeight-20;       //  
     Text  := '';                              // 
end;

procedure TBlockEnd.ShowContour;  //   
begin
     FImage.Canvas.Pen.Color:=RGB($00,$00,$00);
     FImage.Canvas.Brush.Color:=FColor;
     FImage.Canvas.RoundRect(FRect.Left,FRect.Top,FRect.Right,FRect.Bottom,FHeight,FHeight);
end;


////////////////////////////////////////////////////////////////
//                 "" TBlockInput               //
////////////////////////////////////////////////////////////////

procedure TBlockInput.SetPoints;  //  
const  D = 15;  // 
begin
     FPoints[0].X:=FRect.Left+D;   FPoints[0].Y:=FRect.Top;
     FPoints[1].X:=FRect.Right;    FPoints[1].Y:=FRect.Top;
     FPoints[2].X:=FRect.Right-D;  FPoints[2].Y:=FRect.Bottom;
     FPoints[3].X:=FRect.Left;     FPoints[3].Y:=FRect.Bottom;
end;

constructor TBlockInput.Create(I:TImage);  //  
begin
     inherited;                     //  TElem.Create
     FColor:=RGB($98,$FF,$98);      //  
     Text  := ' X';             //   
end;


////////////////////////////////////////////////////////////////
//              "" TBlockAction             //
////////////////////////////////////////////////////////////////

constructor TBlockAction.Create(I:TImage);  //  
begin
     inherited;                     //  TElem.Create
     FColor:=RGB($EE,$EE,$90);      //  
     Text  := 'Y=F(X)';             //   
end;


////////////////////////////////////////////////////////////////
//                 "" TBlockIf               //
////////////////////////////////////////////////////////////////

procedure TBlockIf.SetPoints;  //  
begin
     FPoints[0].X:=FRect.Left;   FPoints[0].Y:=Yc;
     FPoints[1].X:=Xc;           FPoints[1].Y:=FRect.Top;
     FPoints[2].X:=FRect.Right;  FPoints[2].Y:=Yc;
     FPoints[3].X:=Xc;           FPoints[3].Y:=FRect.Bottom;
end;

procedure TBlockIf.ShowContour;  //   
begin
     inherited;                     //  TElem.ShowContour
     FImage.Canvas.Brush.Color:=clWhite;
     if FLinks[1]='2' then FImage.Canvas.TextOut(FRect.Left-10,Yc-20,'');
     if FLinks[1]='4' then FImage.Canvas.TextOut(FRect.Left-10,Yc-20,'');
     if FLinks[2]='2' then FImage.Canvas.TextOut(Xc+10,FRect.Top-10,'');
     if FLinks[2]='4' then FImage.Canvas.TextOut(Xc+10,FRect.Top-10,'');
     if FLinks[3]='2' then FImage.Canvas.TextOut(FRect.Right+10-FImage.Canvas.TextWidth(''),Yc-20,'');
     if FLinks[3]='4' then FImage.Canvas.TextOut(FRect.Right+10-FImage.Canvas.TextWidth(''),Yc-20,'');
     if FLinks[4]='2' then FImage.Canvas.TextOut(Xc+10,FRect.Bottom,'');
     if FLinks[4]='4' then FImage.Canvas.TextOut(Xc+10,FRect.Bottom,'');
end;

constructor TBlockIf.Create(I:TImage);  //  
begin
     inherited;                     //  TElem.Create
     FLinks:='2140';                //  ,    
     FColor:=RGB($FF,$C0,$CB);      //  
     Text  := '?';                  //   
end;


////////////////////////////////////////////////////////////////
//                "" TBlockCircle               //
////////////////////////////////////////////////////////////////

procedure TBlockCircle.SetPoints;  //  
begin
     SetLength(FPoints,6);
     FPoints[0].X:=FRect.Left;      FPoints[0].Y:=Yc;
     FPoints[1].X:=FRect.Left+10;   FPoints[1].Y:=FRect.Top;
     FPoints[2].X:=FRect.Right-10;  FPoints[2].Y:=FRect.Top;
     FPoints[3].X:=FRect.Right;     FPoints[3].Y:=Yc;
     FPoints[4].X:=FRect.Right-10;  FPoints[4].Y:=FRect.Bottom;
     FPoints[5].X:=FRect.Left+10;   FPoints[5].Y:=FRect.Bottom;
end;

constructor TBlockCircle.Create(I:TImage);  //  
begin
     inherited;                     //  TElem.Create
     FLinks:='0304';                //    
     FColor:=RGB($99,$D9,$EA);      //  
     Text  := 'i = i + 1';          //   
end;


////////////////////////////////////////////////////////////////
//               THorLink             //
////////////////////////////////////////////////////////////////

procedure THorLink.SetPoints;  //  
begin
     SetLength(FPoints,6);
     FPoints[0].X:=FRect.Left;      FPoints[0].Y:=Yc;
     FPoints[1].X:=FRect.Left;      FPoints[1].Y:=FRect.Top;
     FPoints[2].X:=FRect.Right;     FPoints[2].Y:=FRect.Top;
     FPoints[3].X:=FRect.Right;     FPoints[3].Y:=Yc;
     FPoints[4].X:=FRect.Right;     FPoints[4].Y:=FRect.Bottom;
     FPoints[5].X:=FRect.Left;      FPoints[5].Y:=FRect.Bottom;
end;

procedure THorLink.ShowContour;  //  
begin
     FImage.Canvas.Pen.Color:=RGB($00,$00,$00);
     FImage.Canvas.MoveTo(FRect.Left,Yc);  FImage.Canvas.LineTo(FRect.Right,Yc);
end;

constructor THorLink.Create(I:TImage);  //  
begin
     inherited;                     //  TElem.Create
     FLinks:='0000';                //  
     Height:=20;                    // 
end;


////////////////////////////////////////////////////////////////
//                 TVerLink             //
////////////////////////////////////////////////////////////////

procedure TVerLink.SetPoints;  //  
begin
     SetLength(FPoints,6);
     FPoints[0].X:=FRect.Left;      FPoints[0].Y:=FRect.Top;
     FPoints[1].X:=Xc;              FPoints[1].Y:=FRect.Top;
     FPoints[2].X:=FRect.Right;     FPoints[2].Y:=FRect.Top;
     FPoints[3].X:=FRect.Right;     FPoints[3].Y:=FRect.Bottom;
     FPoints[4].X:=Xc;              FPoints[4].Y:=FRect.Bottom;
     FPoints[5].X:=FRect.Left;      FPoints[5].Y:=FRect.Bottom;
end;

procedure TVerLink.ShowContour;  //  
begin
     FImage.Canvas.Pen.Color:=RGB($00,$00,$00);
     FImage.Canvas.MoveTo(Xc,FRect.Top);  FImage.Canvas.LineTo(Xc,FRect.Bottom);
end;

constructor TVerLink.Create(I:TImage);  //  
begin
     inherited;                     //  TElem.Create
     FLinks:='0000';                //  
     Width :=20;                    // 
     Height:=100;                   // 
end;


////////////////////////////////////////////////////////////////
//                 - TFlowChart                 //
////////////////////////////////////////////////////////////////

function TFlowChart.FindElem(X,Y:longint):TElem;  //  
var  i :longint;  // 
begin
     result:=nil;
     for i:=0 to Length(FElems)-1 do
         if (FElems[i]<>nil) and (FElems[i].Include(Point(X,Y))) then result:=FElems[i];
end;

procedure TFlowChart.TextChange(Sender: TObject);  //  
begin
     if FCurElem<>nil then FCurElem.Text:=FEdit.Text;
end;

procedure TFlowChart.FixGrid;  //  -  
var  i :longint;  // 
begin
     for i:=0 to Length(FElems)-1 do FElems[i].FixGrid;
end;

procedure TFlowChart.MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);  //  
begin
     if (FCurElem<>nil) and (FCurElem.FPoint>=0) then begin  //   
        FDX:=(FCurElem.Width  div 2) - Abs(FCurElem.FPoints[FCurElem.FPoint].X-FCurElem.Xc);
        FDY:=(FCurElem.Height div 2) - Abs(FCurElem.FPoints[FCurElem.FPoint].Y-FCurElem.Yc);
        FMode:=mdSize;
     end else begin        //    
        FCurElem:=nil;  if FEdit<>nil then FEdit.Text:='';  FCurElem:=FindElem(X,Y);  Show;
        if FCurElem=nil then begin       //  -,    
           FDX:=X;  FDY:=Y;  FMode:=mdChart;
        end else begin
           if FCurElem<>nil then begin   //   
              if FEdit<>nil then begin  FEdit.Text:=FCurElem.Text;  FEdit.SelStart:=Length(FEdit.Text);  end;
              FDX:=FCurElem.Left-X;  FDY:=FCurElem.Top-Y;  FMode:=mdMove;
           end;
        end;
     end;
end;

procedure TFlowChart.MouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);  // 
var  i :longint;  // 
begin
     if FCurElem<>nil then begin         //      
        if FCurElem.NumPoint(Point(X,Y))<0 then FImage.Cursor:=crDefault else FImage.Cursor:=crCross;
     end else FImage.Cursor:=crDefault;
     Case FMode of
       mdMove  :begin      //  
                   FCurElem.Left:=X+FDX;  FCurElem.Top:=Y+FDY;
                end;
       mdSize  :begin      //   
                   i:=Abs(FCurElem.Xc-X)+FDX;  FCurElem.Left:=FCurElem.Xc-i;  FCurElem.Width:=2*i;
                   i:=Abs(FCurElem.Yc-Y)+FDY;  FCurElem.Top:=FCurElem.Yc-i;   FCurElem.Height:=2*i;
                end;
       mdChart :begin      //  
                   for i:=0 to Length(FElems)-1 do begin
                       FElems[i].Left:=FElems[i].Left+(X-FDX);
                       FElems[i].Top:=FElems[i].Top+(Y-FDY);
                   end;
                   FDX:=X;  FDY:=Y;
                end;
     end;
     if FMode<>mdNone then Show;
end;

procedure TFlowChart.MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);  //  
begin
     FMode:=mdNone;  FDX:=0;  FDY:=0;
     if FCurElem=nil then FixGrid else FCurElem.FixGrid;  Show;
end;

procedure TFlowChart.SetImage(I:TImage);  //  
var  j :longint;  // 
begin
     FImage:=I;   if I=nil then Exit;
     FImage.OnMouseDown:=MouseDown;
     FImage.OnMouseMove:=MouseMove;
     FImage.OnMouseUp:=MouseUp;
     for j:=0 to Length(FElems)-1 do FElems[j].Image:=FImage;
     Show;
end;

procedure TFlowChart.SetEdit(E:TEdit);  //     
begin
     FEdit:=E;   if E<>nil then FEdit.OnChange:=TextChange;
end;

////////////////////////////////////////////////////////////////

procedure TFlowChart.Clear;  //  -
var  j :longint;  // 
begin
     for j:=0 to Length(FElems)-1 do FElems[j].Free;
     SetLength(FElems,0);   Show;
end;

constructor TFlowChart.Create;  //  -
begin
     SetLength(FElems,0);
     FImage:=nil;
     FCurElem:=nil;
     FDX:=0;
     FDY:=0;
     FMode:=mdNone;
     FEdit:=nil;
end;

destructor TFlowChart.Destroy;  //  -
begin
     Clear;
end;

procedure TFlowChart.Show;  //  -
var  j :longint;  // 
begin         
     //  
     if FImage=nil then Exit;
     FImage.Canvas.Brush.Color:=RGB($FF,$FF,$FF);
     FImage.Canvas.FillRect(FImage.ClientRect);
     //  
     for j:=0 to Length(FElems)-1 do FElems[j].Show;
     if FCurElem<>nil then FCurElem.ShowSelect;
end;

procedure TFlowChart.Add(B:TElem);  //  
var  L :longint;  //  
begin
     if B=nil then Exit;  B.FixGrid;  B.FImage:=FImage;
     L:=Length(FElems);  SetLength(FElems,L+1);  FElems[L]:=B;
     if (not (B is TBlockBegin)) and (not (B is TBlockEnd)) then begin
        FDX:=-(B.FWidth div 2);  FDY:=-(B.FHeight div 2);
        FMode:=mdMove;
     end;
     FCurElem:=FElems[L];  Show;
end;

procedure TFlowChart.DelSelect;  //   
var  i,j :longint;  // 
begin
     if FCurElem=nil then Exit;   j:=-1;
     for i:=0 to Length(FElems)-1 do if FElems[i]=FCurElem then j:=i;
     if j<0 then Exit;  FElems[j].Free;  FCurElem:=nil;
     for i:=j to Length(FElems)-2 do FElems[i]:=FElems[i+1];
     SetLength(FElems,Length(FElems)-1);
     if FEdit<>nil then FEdit.Text:='';  Show;
end;

procedure TFlowChart.NextMode;  //    
begin
     if FCurElem=nil then Exit;
     if FCurElem.FLinks='0102' then FCurElem.FLinks:='2010' else   //  
     if FCurElem.FLinks='2010' then FCurElem.FLinks:='0201' else
     if FCurElem.FLinks='0201' then FCurElem.FLinks:='1020' else
     if FCurElem.FLinks='1020' then FCurElem.FLinks:='1002' else
     if FCurElem.FLinks='1002' then FCurElem.FLinks:='0012' else   //   
     if FCurElem.FLinks='0012' then FCurElem.FLinks:='2100' else
     if FCurElem.FLinks='2100' then FCurElem.FLinks:='0120' else
     if FCurElem.FLinks='0120' then FCurElem.FLinks:='0102' else
     if FCurElem.FLinks='0304' then FCurElem.FLinks:='1324' else   // 
     if FCurElem.FLinks='1324' then FCurElem.FLinks:='2314' else
     if FCurElem.FLinks='2314' then FCurElem.FLinks:='0304' else
     if FCurElem.FLinks='2140' then FCurElem.FLinks:='4120' else   // 
     if FCurElem.FLinks='4120' then FCurElem.FLinks:='0124' else
     if FCurElem.FLinks='0124' then FCurElem.FLinks:='0142' else
     if FCurElem.FLinks='0142' then FCurElem.FLinks:='2104' else
     if FCurElem.FLinks='2104' then FCurElem.FLinks:='4102' else
     if FCurElem.FLinks='4102' then FCurElem.FLinks:='2140';
     Show;
end;


initialization
    FlowChart:=TFlowChart.Create;    //   
finalization
    FlowChart.Free;                  //   
end.
